%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to reconstruct a cell cycle regulatory subnetwork of 
% Saccharomyces cerevisiae by using BANJO software and to compute the 
% corresponding  performance indexes, assuming different levels of prior 
% knowledge (PK) about the original network structure.
% To this aim, the following functions have been used:
% banjo_results.m and ppv_sens_spec.m.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load a cell array containig the standard (first column) and systematic 
% (second column) names of genes that compose the 
% subnetwork
load Bio_Example_data_results\name_genes_subnetwork

% load the data 
load Bio_Example_data_results\expression_data_subnetwork

% load the original network to identify, obtained by the BioGRID database, 
% taking into account the information of Simon et al  (2001) and Bahler 
% (2005). 
load Bio_Example_data_results\true_subnetwork


% define the number of nodes of the subnetwork
n_nodes=size(original_network,1);

n_best_net=10; % number of the highest scoring networks


% define the sparsity coefficient for the subnetwork
spar_coef=(n_nodes^2-(nnz(original_network(1:27,1:27))-...
    nnz(diag(original_network(1:27,1:27)))))/(n_nodes^2-n_nodes);


% number of tests: we assuming different levels of prior
% knowledge (10%, 20%, 30%, 40% PK) at different levels of sensitivity.
n_tests=4;

% load a cell array containing the Constraints Matrices
load Bio_Example_data_results\Constr_mtx_array

% allocate a memory space to save the output variables 
A_id_cell_Banjo{n_tests}=[];% cell array containing the top-scoring 
                            % networks for each test 
                                         


% define a flag (do_sim), true if you choose to do the simulation,
% otherwise you can load the files.mat, where the identified variables have 
% been saved
do_sim=false;                                     
                                    
if not(do_sim)
    % load the results of the simulations 
    load Bio_Example_data_results\res_bio_banjo       
end



% define matrices to store the results 
PPV_Banjo=zeros(n_best_net,n_tests);
Se_Banjo=zeros(n_best_net,n_tests);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%     COMPUTE THE PERFORMANCE OF BANJO   %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for num_test=1:n_tests
    % num_test=1, corresponding to the results obtained by using 10% of
    % edges a priori known
    % num_test=2, corresponding to the results obtained by using 20% of
    % edges a priori known
    % num_test=3, corresponding to the results obtained by using 30% of
    % edges a priori known
    % num_test=4, corresponding to the results obtained by using 40% of
    % edges a priori known
    
    if num_test==1
        str_pk='_pk_10';
    elseif num_test==2
        str_pk='_pk_20';
    elseif num_test==3
        str_pk='_pk_30';
    elseif num_test==4
        str_pk='_pk_40';
    end    
    
    if do_sim
        % Compute the top-scoring networks corresponding to the
        % connectivity matrices
        eval(['dos(''java -jar banjo.jar settingsFile=data\'...
            'release2.0\dynamic\Banjo_dyn_sett_file_Bio_Example\'...
            'dyn_sett_sys_Bio_Example',str_pk,'.txt'')']);
        % define a string containing the path of the report file
        % returned by Banjo
        filename=['data\release2.0\dynamic\output'...
            '\Banjo_results_Bio_Example'...
            '\results_Bio_Example',str_pk,'.txt'];
        
        % Obtain the connectivity matrices, corresponding to the highest 
        % scoring networks returned by report file of Banjo 
        [A_id_best_Banjo]=banjo_results(filename,n_nodes,...
                n_best_net,false);

        % store the cell array that contains the best identified
        % connectivity matrices for each test
        A_id_cell_Banjo{num_test}=A_id_best_Banjo;
    else
        % Assign the connectivity matrices, corresponding to the highest 
        % scoring networks obtained by Banjo, for each test
        A_id_best_Banjo=A_id_cell_Banjo{num_test};        
    end
        
    % evaluation of the performance for the ten highest scoring networks by 
    % calculating the PPV and sensitivity indexes
    for num_iter=1:n_best_net; 

        Perf_idx= ppv_sens_spec(original_network,...
            A_id_best_Banjo{num_iter});

        % for the performance evaluation, consider the case of directed
        % graph representations: the network is represented as a
        % directed graph, the performance indexes (PPV, Sensitivity) do
        % not take into account the estimated weight of the edge, but
        % only its existence;
        % store in two matrix (PPV_pa, Se_pa) the PPV and Se performance
        % indexes: the row index corresponds to the iteration number of the
        % iterative procedure, the column index corresponds to the kind of 
        % test
        PPV_Banjo(num_iter,num_test)=Perf_idx(2,1);
        Se_Banjo(num_iter,num_test)=Perf_idx(2,2);
        
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%  PLOT RESULTS   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);


% Compute the performance of random algorithm taking into account 10% of 
% edges a priori known
PK_l=0.1;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_10=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_10=x_pr_v*spar_coef;
FP_dir_10=(1-x_pr_v)*spar_coef;
FN_dir_10=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_10=TP_dir_10./(TP_dir_10+FN_dir_10);
% spec_dir_10=TN_dir./(TN_dir_10+FP_dir_10);
PPV_dir_10=TP_dir_10./(TP_dir_10+FP_dir_10);



% Compute the performance of random algorithm taking into account 20% of 
% edges a priori known
PK_l=0.2;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_20=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_20=x_pr_v*spar_coef;
FP_dir_20=(1-x_pr_v)*spar_coef;
FN_dir_20=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_20=TP_dir_20./(TP_dir_20+FN_dir_20);
% spec_dir_20=TN_dir./(TN_dir_20+FP_dir_20);
PPV_dir_20=TP_dir_20./(TP_dir_20+FP_dir_20);


% Compute the performance of random algorithm taking into account 30% of 
% edges a priori known
PK_l=0.3;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_30=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_30=x_pr_v*spar_coef;
FP_dir_30=(1-x_pr_v)*spar_coef;
FN_dir_30=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_30=TP_dir_30./(TP_dir_30+FN_dir_30);
% spec_dir_30=TN_dir./(TN_dir_30+FP_dir_30);
PPV_dir_30=TP_dir_30./(TP_dir_30+FP_dir_30);



PK_l=0.4;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_40=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_40=x_pr_v*spar_coef;
FP_dir_40=(1-x_pr_v)*spar_coef;
FN_dir_40=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_40=TP_dir_40./(TP_dir_40+FN_dir_40);
% spec_dir_40=TN_dir./(TN_dir_40+FP_dir_40);
PPV_dir_40=TP_dir_40./(TP_dir_40+FP_dir_40);

figure
hold on;
grid on
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
axis([0.1 1 0.1 1]);


% plot the performance of BANJO and of the random algorithm 
% assuming 4 different levels of PK:

% 1) 10% of PK
plot(Se_Banjo(:,1),PPV_Banjo(:,1),'dk','MarkerSize',8,...
    'LineWidth',2);
plot(sens_dir_10,PPV_dir_10,'r:','LineWidth',2);

% 2) 20% of PK
plot(Se_Banjo(:,2),PPV_Banjo(:,2),'dk','MarkerSize',8,...
    'LineWidth',2);
plot(sens_dir_20,PPV_dir_20,'r:','LineWidth',2);

% 3) 30% of PK
plot(Se_Banjo(:,3),PPV_Banjo(:,3),'dk','MarkerSize',8,...
    'LineWidth',2);
plot(sens_dir_30,PPV_dir_30,'r:','LineWidth',2);

% 4) 40% of PK
plot(Se_Banjo(:,4),PPV_Banjo(:,4),'dk','MarkerSize',8,...
    'LineWidth',2);
plot(sens_dir_40,PPV_dir_40,'r:','LineWidth',2);

leg1=legend('RANDOM','BANJO');