%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to reconstruct a cell cycle regulatory subnetwork of 
% Saccharomyces cerevisiae. Such network is composed of 27 genes: ten genes 
% that encoded the corresponding transcription factors proteins (ace2, 
% fkh1, swi4, swi5, mbp1, swi6, mcm1, fkh2, ndd1, yox1) and seventeen genes 
% that encoded corresponding cyclin and cyclin/CDK regulator proteins 
% (cln1, cln2, cln3, cdc20, clb1, clb2, clb4, clb5, clb6, sic1, far1, 
% spo12, apc1, tem1, gin4, swe1 and whi5). We have used the microarray data 
% from Spellman et al 1998 and selected the data set produced by alfa 
% factor arrest method. Thus we have obtained a data set of 27 genes and 18 
% data points. The regulatory network between the 27 genes was drawn from 
% the BioGRID database, taking into account the information of Simon et al 
% (2001) and Bahler (2005). To reconstuct the network and to compute the 
% performance indexes, the following functions have been used:
% main_pactls_st.m that recalls ls_ctls_zero_st.m and ls_ctls_full_st.m ; 
% smth_int_data.m; ppv_sens_spec.m.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load a cell array containig the standard (first column) and systematic 
% (second column) names of genes that compose the 
% subnetwork
load Bio_Example_data_results\name_genes_subnetwork

% load the data 
load Bio_Example_data_results\expression_data_subnetwork

% load the original network to identify, obtained by the BioGRID database, 
% taking into account the information of Simon et al  (2001) and Bahler 
% (2005). 
load Bio_Example_data_results\true_subnetwork



% define the maxinum number of iterations for each identification
max_iter_num_pk=10;

% define the number of nodes of the subnetwork
n_nodes=size(original_network,1);


% Apply a smoothing technique in order to filter the measurement noise
% and increase by interpolation the number of observations
Meas_mtx_smth=smth_int_data(Meas_mtx_dec,2,0);

% define the sparsity coefficient for the subnetwork
spar_coef=(n_nodes^2-(nnz(original_network(1:27,1:27))-...
    nnz(diag(original_network(1:27,1:27)))))/(n_nodes^2-n_nodes);

% Assume that B vector is null
B_vct=zeros(n_nodes,1);

% number of tests: we assuming different levels of prior
% knowledge (10%, 20%, 30%, 40% PK) at different levels of sensitivity.
n_tests=4;

% load a cell array containing the Constraints Matrices
load Bio_Example_data_results\Constr_mtx_array


% allocate a memory space to save the output variables of the iterative
% procedure
A_id_cell_pactls{n_tests}=[]; % cell array containing the identified 
                          % connectivity matrices, obtained by
                          % using PACTLS, for each test  

% U_id_cell_pactls{n_tests}=[]; % cell array containing the identified input 
                            % vectors, obtained by
                            % using PACTLS, for each test  

est_err_cell_pactls{n_tests}=[]; % cell array containing the errors of the 
                             % identified models, obtained by
                             % using PACTLS, for each test 
                             
sol_found_pactls{n_tests}=[]; % cell array containing a flag, true if a 
                          % solution has been found, obtained by
                          % using PACTLS, for each test  

% define a flag (do_sim), true if you choose to do the simulation,
% otherwise you can load the files.mat, where the identified variables have 
% been saved
do_sim=false;   

if not(do_sim)
    load Bio_Example_data_results\res_bio_pactls
end                                

% define matrices to store the results 
PPV_pactls=zeros(max_iter_num_pk,n_tests);
Se_pactls=zeros(max_iter_num_pk,n_tests);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%    COMPUTE THE PERFORMANCE OF PACTLS     %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for num_test=1:n_tests
    
    % num_test=1, corresponding to the results obtained by using 10% of
    % edges a priori known
    % num_test=2, corresponding to the results obtained by using 20% of
    % edges a priori known
    % num_test=3, corresponding to the results obtained by using 30% of
    % edges a priori known
    % num_test=4, corresponding to the results obtained by using 40% of
    % edges a priori known
    
    if do_sim
        % Compute the variables to be identified (the connectivity
        % matrix A and the exogenous perturbation vector B)

        % Inference by using PACTLS
        [A_id_cell,U_id_cell,est_err_cell,sol_found] = ...
            main_pactls_st(Meas_mtx_smth,B_vct,Constr_mtx_pk{num_test},...
            n_nodes,max_iter_num_pk);

        % store the identified variables
        A_id_cell_pactls{num_test}=A_id_cell;
%         U_id_cell_pactls{num_test}=U_id_cell;
        est_err_cell_pactls{num_test}=est_err_cell;
        sol_found_pactls{num_test}=sol_found;
    else
        A_id_cell=A_id_cell_pactls{num_test};
        % U_id_cell=U_id_cell_pactls{num_test};
        est_err_cell=est_err_cell_pactls{num_test};
    end

    % evaluation of the performance at different levels of Sensitivity (Se)
    for num_iter=1:max_iter_num_pk+1
        if num_iter<=length(A_id_cell)
            if num_iter==1
                Perf_idx= ppv_sens_spec(original_network,A_id_cell{num_iter}(1:27,1:27));
            else
                Perf_idx= ppv_sens_spec(original_network,A_id_cell{num_iter});

            end

            % for the performance evaluation, consider the case of directed
            % graph representations: the network is represented as a
            % directed graph, the performance indexes (PPV, Sensitivity) do
            % not take into account the estimated weight of the edge, but
            % only its existence; store in two matrix (PPV_pactls, Se_pactls) the
            % PPV and Se performance indexes: the row index corresponds to
            % the iteration number of the iterative procedure, the column
            % index corresponds to the test
            PPV_pactls(num_iter,num_test)=Perf_idx(2,1);
            Se_pactls(num_iter,num_test)=Perf_idx(2,2);

        end
    end

    if do_sim
        eval(['save Bio_Example_data_results\res_bio_pactls  A_id_cell_pactls est_err_cell_pactls ',...
            ' sol_found_pactls '])
    end

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%  PLOT RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);


% Compute the performance of random algorithm taking into account 10% of 
% edges a priori known
PK_l=0.1;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_10=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_10=x_pr_v*spar_coef;
FP_dir_10=(1-x_pr_v)*spar_coef;
FN_dir_10=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_10=TP_dir_10./(TP_dir_10+FN_dir_10);
% spec_dir_10=TN_dir./(TN_dir_10+FP_dir_10);
PPV_dir_10=TP_dir_10./(TP_dir_10+FP_dir_10);



% Compute the performance of random algorithm taking into account 20% of 
% edges a priori known
PK_l=0.2;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_20=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_20=x_pr_v*spar_coef;
FP_dir_20=(1-x_pr_v)*spar_coef;
FN_dir_20=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_20=TP_dir_20./(TP_dir_20+FN_dir_20);
% spec_dir_20=TN_dir./(TN_dir_20+FP_dir_20);
PPV_dir_20=TP_dir_20./(TP_dir_20+FP_dir_20);


% Compute the performance of random algorithm taking into account 30% of 
% edges a priori known
PK_l=0.3;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_30=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_30=x_pr_v*spar_coef;
FP_dir_30=(1-x_pr_v)*spar_coef;
FN_dir_30=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_30=TP_dir_30./(TP_dir_30+FN_dir_30);
% spec_dir_30=TN_dir./(TN_dir_30+FP_dir_30);
PPV_dir_30=TP_dir_30./(TP_dir_30+FP_dir_30);


PK_l=0.4;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_40=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_40=x_pr_v*spar_coef;
FP_dir_40=(1-x_pr_v)*spar_coef;
FN_dir_40=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_40=TP_dir_40./(TP_dir_40+FN_dir_40);
% spec_dir_40=TN_dir./(TN_dir_40+FP_dir_40);
PPV_dir_40=TP_dir_40./(TP_dir_40+FP_dir_40);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% PLOT RESULTS OF PACTLS %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure
hold on;
grid on
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
col_p='b';

for num_test=1:4    
    hold on
    grid on
    if num_test==1
        plot(sens_dir_10,PPV_dir_10,'r-','LineWidth',2);
    elseif num_test==2
        plot(sens_dir_20,PPV_dir_20,'r-','LineWidth',2);
    elseif num_test==3
       plot(sens_dir_30,PPV_dir_30,'r-','LineWidth',2);
    elseif num_test==4
        plot(sens_dir_40,PPV_dir_40,'r-','LineWidth',2);
    end
    plot(Se_pactls(:,num_test),PPV_pactls(:,num_test),'MarkerFaceColor',col_p,'MarkerEdgeColor',col_p,...
        'MarkerSize',8,'Marker','o','LineWidth',2,'LineStyle','none');
end
leg1=legend('RANDOM','PACTLS');
