%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to compute the reconstruced connectivity matrices,
% the perturbation vectors B and performance indexes for the in-silico 
% networks of ten and twenty nodes, generated by using the scalefree 
% network generation model proposed by Albert and Barabasi, via PACTLS 
% algorithm. 
% For each network, different noise levels and different levels of prior 
% knowledge (PK) about the original network structure have been assumed.
% To this aim, the following  functions have been used:
% main_pactls_st.m that recalls ls_ctls_zero_st.m and ls_ctls_full_st.m ; 
% ppv_sens_spec.m; estim_err_thr.m; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc
clear all
% close all


% define the maxinum number of iterations for each identification
max_iter_num=10;

% threshold value for the estimation error 
thr_err=0.1;

net_nodes=20; % test over the networks of twenty nodes 

% networks number
n_networks=20;

% number of tests: for each network, we consider a set of data of 25 time 
% points, assuming two levels of noise (10%, 30%) and two levels of 
% prior knowledge (PK) (10% and 20%)
n_tests=4;

% allocate a memory space to save the output variables of the iterative
% procedure
A_id_cell_pactls{n_networks,n_tests}=[]; % cell array containing the identified 
                                     % connectivity matrices, obtained by
                                     % using PACTLS, for each network and 
                                     % test  
                                  
U_id_cell_pactls{n_networks,n_tests}=[]; % cell array containing the identified 
                                     % input vectors, obtained by
                                     % using PACTLS, for each network and 
                                     % test
                                     
est_err_cell_pactls{n_networks,n_tests}=[]; % cell array containing the errors 
                                        % of the identified models,obtained
                                        % by using PACTLS, for each 
                                        % network and test  
                                     
sol_found_pactls{n_networks,n_tests}=[]; % cell array containing a flag, true 
                                     % if a solution has been found, 
                                     % obtained by using PACTLS, for each
                                     % network and test   

% define a flag (do_sim), true if you choose to do the simulation,
% otherwise you can load the files.mat, where the identified variables have 
% been saved
do_sim=false;               

if not(do_sim)
    eval(['load results_net_in_silico_20n\res_pactls_',num2str(net_nodes),'n'])
end
                                    
% define a vector to store the sparsity coefficient for each network
spars_networks=zeros(1,n_networks);

% define matrices to store the results 
PPV_pactls=zeros(n_networks,n_tests);
Se_pactls=zeros(n_networks,n_tests);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%    COMPUTE THE PERFORMANCE OF PACTLS and PACTLS_noPA    %%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for num_net=1:n_networks
    % load the data for the current network
    
    eval(['load data_net_in_silico_20n\sys_in_silico_20n_net_',...
        num2str(num_net)]);
    spars_networks(num_net)=spar_coef;
    
    for num_test=1:n_tests
        % num_test=1, corresponding to the results obtained by using a data
        %             set of 25 time points, 10% level of noise, 10% level of
        %             prior knowledge             
        % num_test=2, corresponding to the results obtained by using a data
        %             set of 25 time points, 30% level of noise, 10% level of
        %             prior knowledge 
        % num_test=3, corresponding to the results obtained by using a data
        %             set of 25 time points, 10% level of noise, 20% level of
        %             prior knowledge 
        % num_test=4, corresponding to the results obtained by using a data
        %             set of 25 time points, 30% level of noise, 20% level of
        %             prior knowledge
        
        if rem(num_test,2)
            str='noise10';
            Meas_mtx=Meas_mtx_25tp_noise10; % each column contains the
            % measurements (25 time points) with 10% noise level for a node 
        else
            str='noise30';
            Meas_mtx=Meas_mtx_25tp_noise30; % each column contains the
            % measurements (25 time points) with 30% noise level for a node
        end
        
        if num_test>=1 && num_test<=2  
            sign_constr_list=Constr_mtx_10;
        elseif num_test>2 && num_test<=4
            sign_constr_list=Constr_mtx_20;
        end
        if do_sim
            % Compute the variables to be identified (the connectivity 
            % matrix A and the exogenous perturbation vector B)

            % Inference by using PACTLS
            [A_id_cell,U_id_cell,est_err_cell,sol_found] = ...
                main_pactls_st(Meas_mtx,B_vct,sign_constr_list,...
                n_nodes,max_iter_num);
            
            % store the identified variables
            A_id_cell_pactls{num_net,num_test}=A_id_cell;
            U_id_cell_pactls{num_net,num_test}=U_id_cell;
            est_err_cell_pactls{num_net,num_test}=est_err_cell;
            sol_found_pactls{num_net,num_test}=sol_found;
        else
            A_id_cell=A_id_cell_pactls{num_net,num_test};
            % U_id_cell=U_id_cell_pactls{num_net,num_test};
            est_err_cell=est_err_cell_pactls{num_net,num_test};
        end
        
        % selection of the network, taking into account the norm of vector
        % of residuals
        idx_sel=estim_err_thr(est_err_cell,A_id_cell,thr_err);
        % evaluation of the performance by calculating the PPV and
        % sensitivity indexes
        if not(isnan(idx_sel))
            Perf_idx= ppv_sens_spec(original_network,A_id_cell{idx_sel});
        
            % for the performance evaluation, consider the case of directed
            % graph representations: the network is represented as a
            % directed graph, the performance indexes (PPV, Sensitivity) do
            % not take into account the estimated weight of the edge, but
            % only its existence;
            % save in two matrix (PPV_pactls, Se_pactls) the PPV and Sensitivity
            % (Se) performance indexes: the row index corresponds to the
            % network number, the column index corresponds to the kind of
            % test
            PPV_pactls(num_net,num_test)=Perf_idx(2,1);
            Se_pactls(num_net,num_test)=Perf_idx(2,2);
        elseif not(isempty(A_id_cell_pactls{num_net,num_test}{1}))
            Perf_idx= ppv_sens_spec(original_network,A_id_cell_pactls{num_net,num_test}{1}(:,1:20));
            PPV_pactls(num_net,num_test)=Perf_idx(2,1);
            Se_pactls(num_net,num_test)=Perf_idx(2,2);
        else
            PPV_pactls(num_net,num_test)=0;
            Se_pactls(num_net,num_test)=0;
        end
      
        if do_sim
            eval(['save results_net_in_silico_20n\res_pactls_',num2str(net_nodes),'n  A_id_cell_pactls U_id_cell_pactls ' ...
                ' est_err_cell_pactls  sol_found_pactls '])
        end        
    end
end


x_pr_v=linspace(0,1,100);

spar_coef_mean=mean(spars_networks);
% Compute the performance of random algorithm taking into account 10% of 
% edges a priori known
PK_l=0.1;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_10=PK_l*(1-spar_coef_mean)+(1-PK_l)*(1-spar_coef_mean)*(1-x_pr_v);
%TN_dir_10=x_pr_v*spar_coef_mean;
FP_dir_10=(1-x_pr_v)*spar_coef_mean;
FN_dir_10=(1-PK_l)*x_pr_v*(1-spar_coef_mean);
sens_dir_10=TP_dir_10./(TP_dir_10+FN_dir_10);
% spec_dir_10=TN_dir./(TN_dir_10+FP_dir_10);
PPV_dir_10=TP_dir_10./(TP_dir_10+FP_dir_10);



% Compute the performance of random algorithm taking into account 20% of 
% edges a priori known
PK_l=0.2;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_20=PK_l*(1-spar_coef_mean)+(1-PK_l)*(1-spar_coef_mean)*(1-x_pr_v);
%TN_dir_20=x_pr_v*spar_coef_mean;
FP_dir_20=(1-x_pr_v)*spar_coef_mean;
FN_dir_20=(1-PK_l)*x_pr_v*(1-spar_coef_mean);
sens_dir_20=TP_dir_20./(TP_dir_20+FN_dir_20);
% spec_dir_20=TN_dir./(TN_dir_20+FP_dir_20);
PPV_dir_20=TP_dir_20./(TP_dir_20+FP_dir_20);


figure
for count=1:4
    s=subplot(2,2,count);
    hold on;
    grid on
    set(s,'FontSize',12)
    axis([0.1 0.4 0.1 1 ]);
    if count==1
        num_test=1; % 10% level of noise, 10% of PK
        title('PK=10%','FontSize',12);
        ylabel(['10% noise',sprintf('\n'),'PPV'],'FontSize',12);
        plot(sens_dir_10,PPV_dir_10,'r-','LineWidth',2);
        axis([0.1 0.4 0.1 1 ]);

    elseif count==2
        num_test=3; % 10% level of noise, 20% of PK
        title('PK=20%','FontSize',12);
        plot(sens_dir_20,PPV_dir_20,'r-','LineWidth',2);
        axis([0.1 0.4 0.1 1 ]);


    elseif count==3
        num_test=2; % 30% level of noise, 10% of PK
        ylabel(['30% noise',sprintf('\n'),'PPV'],'FontSize',12);
        plot(sens_dir_10,PPV_dir_10,'r-','LineWidth',2);
        xlabel('Sn','FontSize',12)
        axis([0.1 0.4 0.1 1 ]);

    elseif count==4
        num_test=4; % 30% level of noise, 20% of PK
        plot(sens_dir_20,PPV_dir_20,'r-','LineWidth',2);
        xlabel('Sn','FontSize',12)
        axis([0.1 0.4 0.1 1 ]);
    end

    col_p='b';
    col_b='k';
    plot(mean(Se_pactls(:,num_test)),mean(PPV_pactls(:,num_test)),'MarkerFaceColor',col_p,'MarkerEdgeColor',col_p,...
        'MarkerSize',8,'Marker','o','LineWidth',2,'LineStyle','none');
%     plot(mean(Se_Banjo(:,num_test)),mean(PPV_Banjo(:,num_test)),'MarkerEdgeColor',col_b,...
%         'MarkerSize',8,'Marker','d','LineWidth',2,'LineStyle','none');
    
    herr1=herrorbar(mean(Se_pactls(:,num_test)),mean(PPV_pactls(:,num_test)),std(Se_pactls(:,num_test)));
    set(herr1,'LineWidth',1);
    errorbar(mean(Se_pactls(:,num_test)),mean(PPV_pactls(:,num_test)),std(PPV_pactls(:,num_test)),'LineWidth',1);
%     herr3=herrorbar(mean(Se_Banjo(:,num_test)),mean(PPV_Banjo(:,num_test)),std(Se_Banjo(:,num_test)),col_b);
%     set(herr3,'LineWidth',1);
%     errorbar(mean(Se_Banjo(:,num_test)),mean(PPV_Banjo(:,num_test)),std(PPV_Banjo(:,num_test)),col_b,'LineWidth',1);
    
    
end

leg1=legend('RANDOM','PACTLS','Orientation','horizontal');
% set(leg1,'OuterPosition',[0.1 0.02 0.4 0.04]);
% set(leg1,'Position',[0.07591 0.02692 0.8808 0.06018]);
% g=[1 1 816 651];
% 
% set(gcf,'OuterPosition',g)

