function [ID_LS ID_CTLS] = ls_ctls_full_st(Meas_mtx,B_vct,sign_constr_list,null_constr_list,num_exp,tag_ctls)

% The function computes the connectivity matrix A and exogenous
% perturbation vector B via LS and CTLS, with imposing the desired zero
% entries of the connectivity matrix A.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INPUT VARIABLES
%
% Meas_mtx:         Measurements matrix, each column contains the 
%                   measurements for a node
%                   
% B_vct:            Vector that models the effect of external perturbations 
%
% sign_constr_list: Matrix with 3 columns.
%             Signed constraints to be imposed on the matrix coefficients
%             first column --> row-index of the constrained coefficient
%             second column --> column-index of the constrained coefficient
%             third column --> sign of the constrained coefficient
%
% null_constr_list: List that contains the coefficients of the connectivity
%                   matrix to be nullified          
%  
% num_exp:          Number of experiments
%
% tag_ctls:         Flag true, if CTLS technique is performed
%
% OUTPUT VARIABLES
%
% ID_LS:      structure containing the reconstructed connectivity matrix A, 
%             the exogenous perturbation vector B, a flag about the
%             stability and the prediction and "dynamic" error, computed 
%             via LS       
%               
% ID_CTLS:    structure containing the reconstructed connectivity matrix A, 
%             the exogenous perturbation vector B, a flag about the
%             stability and the prediction and "dynamic" error, computed 
%             via CTLS       
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin==4
    % Default value of num_exp
    num_exp=1;
    % Default value of tag_ctls
    tag_ctls=1;
elseif nargin==5
    tag_ctls=1;
elseif nargin~=6
    % Wrong number of input arguments
    error('Wrong number of input arguments');
end

tol=0.001;

eps_const=tol;

[n_time_pts,n_nodes]=size(Meas_mtx);
% Matrix of the measurements at time point k from 1 to n_time_pts-1
Omega_mtx=Meas_mtx(1:n_time_pts-1,:)';
Omega_mtx=[Omega_mtx;kron(eye(num_exp),ones(1,n_time_pts-1))];
% Matrix of the measurements at time point k+1 from 2 to n_time_pts
Theta_mtx=Meas_mtx(2:n_time_pts,:)';


% optimisation options for the CTLS
opt_options = optimset('Display','notify','TolX',tol,'TolFun',tol,'TolCon',tol,...
    'MaxFunEvals',4e3,'MaxIter',2e2,'LargeScale','off');

if nnz(B_vct)>=1
    % find the elements of B not equal to zero
    idx_gen_pert=find(B_vct);
    for idx1=1:n_nodes
        in_null_list=true;
        for idx=1:length(idx_gen_pert)
            if idx1==idx_gen_pert(idx)
                in_null_list=false;
            end
        end
        if in_null_list
            null_constr_list_old=null_constr_list;
            null_constr_list=[null_constr_list_old;idx1 n_nodes+1];
        else
            sign_constr_list_old=sign_constr_list;
            sign_constr_list=[sign_constr_list_old;idx1 n_nodes+1 sign(B_vct(idx1))];
        end
    end
else % all the elements of B are equal to zero
    for idx1=1:n_nodes
        null_constr_list_old=null_constr_list;
        null_constr_list=[null_constr_list_old;idx1 n_nodes+1];
    end
end


% Rec_mtx_LS=[];
% Rec_mtx_CTLS=[];

if isempty(null_constr_list)
    Omega_mtx_pinv=pinv(Omega_mtx);
    Rec_mtx_LS= Theta_mtx*Omega_mtx_pinv;
else
    for idx=1:n_nodes
        %find the elements that have to be nullified for each row
        idx_col2del=find(null_constr_list(:,1)==idx);
        if isempty(idx_col2del)
            Omega_mtx_pinv=pinv(Omega_mtx);
            Rec_mtx_LS(idx,:)=Theta_mtx(idx,:)*Omega_mtx_pinv;
        else
            %define a new linear regression, setting to zero the j-th
            %row of Omega
            col2del=null_constr_list(idx_col2del,2);
            Omega_mtx_new=Omega_mtx;
            Omega_mtx_new(col2del,:)=0;
            Omega_mtx_new_pinv=pinv(Omega_mtx_new);
            Rec_mtx_LS(idx,:)=Theta_mtx(idx,:)*Omega_mtx_new_pinv;
            Rec_mtx_LS(idx,col2del)=0;            
        end
    end
end

% Compute the state evolution of the identified model and compare
% with experimental measurements
[err_pr_LS err_dy_LS]=stima_err_pr_dy(Rec_mtx_LS,Meas_mtx,num_exp);

eig_Rec_mtx_LS=eig(Rec_mtx_LS(1:n_nodes,1:n_nodes));
mod_eig_val=abs(eig_Rec_mtx_LS);
if max(mod_eig_val)>=1
    unstbl_mtx_LS=true;
else
    unstbl_mtx_LS=false;
end

% store the identified variables
ID_LS.A=sparse(Rec_mtx_LS(1:n_nodes,1:n_nodes));
ID_LS.B=sparse(Rec_mtx_LS(1:n_nodes,num_exp+n_nodes));
ID_LS.unstb=unstbl_mtx_LS;
ID_LS.err_pr=err_pr_LS;
ID_LS.err_dy=err_dy_LS;


if tag_ctls
    % Perform CTLS
    if isempty(null_constr_list)
        for idx=1:n_nodes

            b_vec=Theta_mtx(idx,:)';
            A_mat=Omega_mtx';
            C_mat=[A_mat b_vec];

            % initial guess from the LS
            x0_CTLS = Rec_mtx_LS(idx,:);

            % bound
            %             lb = x0_CTLS(:)-abs(x0_CTLS(:))*2;
            %             ub = x0_CTLS(:)+abs(x0_CTLS(:))*2;
            lb = x0_CTLS(:)-inf;%-abs(x0_CTLS(:))*2;
            ub = x0_CTLS(:)+inf;%abs(x0_CTLS(:))*2;
            
            % find the sign constraints for each row
            idx_col2constr=find(sign_constr_list(:,1)==idx);
            if not(isempty(idx_col2constr))
                for idx_c=1:length(idx_col2constr)
                    type_cstr=sign_constr_list(idx_col2constr(idx_c),3);
                
                    if (type_cstr==1)
                        lb(sign_constr_list(idx_col2constr(idx_c),2))=eps_const;
                        ub(sign_constr_list(idx_col2constr(idx_c),2))=inf;
                    else
                        lb(sign_constr_list(idx_col2constr(idx_c),2))=-inf;
                        ub(sign_constr_list(idx_col2constr(idx_c),2))=-eps_const;
                    end
                end
            end
            
            % solve
%             x_CTLS = fmincon(@(x_CTLS)cost_CTLS(x_CTLS,idx,C_mat,n_nodes,...
%                 n_time_pts,num_exp),x0_CTLS,[],[],[],[],lb,ub,[],opt_options);
           
            [x_CTLS] = fmincon(@(x_CTLS)cost_CTLS(x_CTLS,idx,C_mat,n_nodes,n_time_pts,num_exp),...
                x0_CTLS,[],[],[],[],lb,ub,@(x_CTLS)mycon_ctls(x_CTLS,n_nodes),opt_options);
           
            Rec_mtx_CTLS(idx,:) = x_CTLS;
        end
    else
        for idx=1:n_nodes

            b_vec=Theta_mtx(idx,:)';
            A_mat=Omega_mtx';
            C_mat=[A_mat b_vec];

            % initial guess from the LS
            x0_CTLS = Rec_mtx_LS(idx,:);
            % bound
            %             lb=[];
            %             ub=[];
            %             lb = x0_CTLS(:)-abs(x0_CTLS(:))*2;
            %             ub = x0_CTLS(:)+abs(x0_CTLS(:))*2;
            lb = x0_CTLS(:)-inf;%abs(x0_CTLS(:))*2;
            ub = x0_CTLS(:)+inf;%abs(x0_CTLS(:))*2;
            
            % find the sign constraints for each row
            idx_col2constr=find(sign_constr_list(:,1)==idx);
            if not(isempty(idx_col2constr))
                for idx_c=1:length(idx_col2constr)
                    type_cstr=sign_constr_list(idx_col2constr(idx_c),3);
                
                    if (type_cstr==1)
                        lb(sign_constr_list(idx_col2constr(idx_c),2))=eps_const;
                        ub(sign_constr_list(idx_col2constr(idx_c),2))=inf;
                    else
                        lb(sign_constr_list(idx_col2constr(idx_c),2))=-inf;
                        ub(sign_constr_list(idx_col2constr(idx_c),2))=-eps_const;
                    end
                end
            end

            %find the elements that have to be nullified for each row
            idx_col2del=find(null_constr_list(:,1)==idx);
            if not(isempty(idx_col2del))
                col2del=null_constr_list(idx_col2del,2);
                lb(col2del)=0;
                ub(col2del)=0;
            end

            % solve
%             x_CTLS = fmincon(@(x_CTLS)cost_CTLS(x_CTLS,idx,C_mat,n_nodes,...
%                 n_time_pts,num_exp),x0_CTLS,[],[],[],[],lb,ub,[],opt_options);
           
            [x_CTLS] = fmincon(@(x_CTLS)cost_CTLS(x_CTLS,idx,C_mat,n_nodes,n_time_pts,num_exp),...
                x0_CTLS,[],[],[],[],lb,ub,@(x_CTLS)mycon_ctls(x_CTLS,n_nodes),opt_options);
            
            Rec_mtx_CTLS(idx,:) = x_CTLS;
            Rec_mtx_CTLS(idx,col2del)=0;
        end
    end

    mod_eig_val=abs(eig(Rec_mtx_CTLS(1:n_nodes,1:n_nodes)));
    if max(mod_eig_val)>=1
        unstbl_mtx_CTLS=true;
    else
        unstbl_mtx_CTLS=false;
    end

    % Compute the state evolution of the identified model and compare
    % with experimental measurements
    [err_pr_CTLS err_dy_CTLS]=stima_err_pr_dy(Rec_mtx_CTLS,Meas_mtx,num_exp);
    
    % store the identified variables
    ID_CTLS.A=sparse(Rec_mtx_CTLS(1:n_nodes,1:n_nodes));
    ID_CTLS.B=sparse(Rec_mtx_CTLS(1:n_nodes,num_exp+n_nodes));
    ID_CTLS.unstb=unstbl_mtx_CTLS;
    ID_CTLS.err_pr=err_pr_CTLS;
    ID_CTLS.err_dy=err_dy_CTLS;
else
    ID_CTLS=[];
end




