function [A_id_cell,U_id_cell,est_err_cell,sol_found] = ...
    main_pactls_st(Meas_mtx,B_vct,sign_constr_list,...
    perc_edge,max_iter_num,th_er)

% Let us assume that a biological network is represented
% by a a stable dynamical linear system, in the form dot(x)= Ax +Bu,
% The function allows to identify  the connectivity matrix A and exogenous
% perturbation vector B from the experimental measurements
% via CTLS technique.
% The initialization network is identified by imposing only selfloops on 
% every vertex.
% Subsequently, new edges are added step-by-step to the network, including 
% those derived by a priori information, according to an edge selection 
% heuristic, derived from the scale-free network generation model proposed 
% by Barabasi and Albert. The iterative procedure stops when the number of 
% the iterations is equal to that maximum.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INPUT VARIABLES
%
% Meas_mtx:     Measurements matrix, each column contains the measurements 
%               for a node
%
% B_vct:        Vector that models the effect of external
%               perturbations 
%
% sign_constr_list: Matrix with 3 columns.
%             Signed constraints to be imposed on the matrix coefficients
%             first column --> row-index of the constrained coefficient
%             second column --> column-index of the constrained coefficient
%             third column --> sign of the constrained coefficient
%             
% perc_edge:    Coefficient that determinates the number of edges that are
%               selected and added to the network at each step
%
% max_iter_num: maximum number of iterations
%
% th_er:        threshold of the error 
%
% OUTPUT VARIABLES
%
% A_id_cell:    Cell array that contains the identified connectivity  
%               matrices at each step 
%
% U_id_cell:    Cell array that contains the identified input vectors at  
%               each step
%
% est_err_cell: Cell array that contains the norm of vector of residuals    
%               generated by the identified model at each step 
%
% sol_found:    Flag true, if a solution has been found 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin==4
    % Default value of maximum number of iterations and threshold of the
    % error
    max_iter_num=10;
    th_er=inf;
elseif nargin==5
    % Default value of the threshold of the error
    th_er=inf;
elseif nargin~=6
    % Wrong number of input arguments
    error('Wrong number of input arguments');
end

% determine the number of time points and nodes,respectively
[n_time_pts,n_nodes]=size(Meas_mtx);

% iteration counter
iter_num=1;

% number of the identifications
ident_num=1;

% number of the added edges
n_link_added=0;



% define matrix which contains the added edges 
if not(isempty(sign_constr_list))
    Constr_mtx=sign_constr_list(:,1:2);
else
    Constr_mtx=[];
end

% define a list of coefficents that determine the instability/(not 
% solution of the problem) of the system (Unstb_list) 
Unstbl_list=[];

          
% first identification:isolated network or by adding only the constraints to
% be imposed
[ID_LS ID_CTLS] = ls_ctls_zero_st(Meas_mtx,B_vct,sign_constr_list,Constr_mtx);

% store the identification
A_id_cell{ident_num}=ID_CTLS.A;
U_id_cell{ident_num}=ID_CTLS.B;
est_err_cell{ident_num}=ID_CTLS.err_dy;

% calculate the preferential attachment function
pref_attach=zeros(1,n_nodes);
for idx2=1:n_nodes
    pref_attach(idx2)=(nnz(ID_CTLS.A(:,idx2)))/...
        (nnz(ID_CTLS.A));
end

% identification without delete no coefficient
[ID_LS_full ID_CTLS_full]=ls_ctls_full_st(Meas_mtx,B_vct,sign_constr_list,[]);

A_id_cell{ident_num}(1:n_nodes,n_nodes+1:2*n_nodes)=ID_CTLS_full.A;
U_id_cell{ident_num}(:,2)=ID_CTLS_full.B;

if ID_CTLS_full.unstb 
    sol_found=false;
    return
% elseif ID_CTLS_full.unstb
%     A_id_temp=ID_LS_full.A;
%     Normal_A_id=zeros(n_nodes,n_nodes);
%     for idx1=1:n_nodes
%          row_norm=norm(A_id_temp(idx1,:));
%          for idx2=1:n_nodes
%              % Normalization of each coefficient with respect to the values on
%              % the same row and column
%              Normal_A_id(idx1,idx2)=abs(A_id_temp(idx1,idx2))/sqrt(row_norm*norm(A_id_temp(:,idx2)));
%          end
%     end
%     % Don't take account the diagonal coefficient and the prior constraints to
%     % generate the edges list to be added
%     for idx2=1:n_nodes
%         Normal_A_id(idx2,idx2)=0;
%     end
%     if not(isempty(Constr_mtx))
%         for idx1=1:size(Constr_mtx,1)
%             Normal_A_id(Constr_mtx(idx1,1),Constr_mtx(idx1,2))=0;
%         end
%     end
%     % Compute the edge ranking matrix (null_list)
%     [idx_row idx_col val_coef]=find(Normal_A_id);
%     [val_coef_sort idx_sort]=sort(val_coef,'ascend');
%     null_list=[idx_row(idx_sort) idx_col(idx_sort) val_coef_sort];
%     num_null=1;
%     unstb_ctls=true;
%     while unstb_ctls && num_null<=size(null_list,1)
%         [ID_LS_full_1 ID_CTLS_full_1]=ls_ctls_full_st(Meas_mtx,B_vct,sign_constr_list,null_list(1:num_null,1:2));
%         num_null=num_null+1;
%         unstb_ctls=ID_CTLS_full_1.unstb;
%     end
%     if not(unstb_ctls)
%         A_id_temp=ID_CTLS_full_1.A;
%         sol_found=true;
%     else
%         sol_found=false;
%         return
%     end
else
    sol_found=true;
    A_id_temp=ID_CTLS_full.A;
end


% Compute the normalized A matrix
Normal_A_id=zeros(n_nodes,n_nodes);
for idx1=1:n_nodes
    row_norm=norm(A_id_temp(idx1,:));
    for idx2=1:n_nodes
        % Normalization of each coefficient with respect to the values on
        % the same row and column
        Normal_A_id(idx1,idx2)=abs(A_id_temp(idx1,idx2))/sqrt(row_norm*norm(A_id_temp(:,idx2)));
    end
end

if not(isempty(sign_constr_list))
    % Compute the normalized weighted A matrix
    Normal_Weight_A_id=zeros(n_nodes,n_nodes);
    for idx1=1:n_nodes
        norm_prob=pref_attach*Normal_A_id(idx1,:)';
        for idx2=1:n_nodes
            Normal_Weight_A_id(idx1,idx2)=...
                Normal_A_id(idx1,idx2)*pref_attach(idx2)/norm_prob;
        end
    end
else
    Normal_Weight_A_id=Normal_A_id;
end

% Don't take account the diagonal coefficient and the prior constraints to 
% generate the edges list to be added  
for idx2=1:n_nodes
    Normal_Weight_A_id(idx2,idx2)=0;
end
if not(isempty(sign_constr_list))
    for idx1=1:size(sign_constr_list,1)
        Normal_Weight_A_id(sign_constr_list(idx1,1),sign_constr_list(idx1,2))=0;
    end
end   

% Compute the edge ranking matrix (Constr_list)
[idx_row idx_col val_coef]=find(Normal_Weight_A_id);
[val_coef_sort idx_sort]=sort(val_coef,'descend');
Constr_list=[idx_row(idx_sort) idx_col(idx_sort) val_coef_sort];


num_it_ins_all_nodes=0;% flag, if >0, then all the edges were added



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%   Algorithm for adding  the coefficients    %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

while   iter_num<=max_iter_num 
    
    % compute the mu_k function, a decreasing function of k (k=iter_num)
    n_link_added_id=ceil(ceil(n_nodes/perc_edge)*n_nodes/(iter_num+1));
    
    % number of edges to be added at k-th iteration
    n_link_added=n_link_added_id+n_link_added;
    
    % define the new list of the edge to be added (new_Constr_mtx) 
    if n_link_added<size(Constr_list,1)
        new_Constr_mtx=Constr_list(1:n_link_added,1:2);
    else
        num_it_ins_all_nodes=1+num_it_ins_all_nodes;
        new_Constr_mtx=Constr_list;
    end
    if num_it_ins_all_nodes>1
        break
    end
    
    % Check if the elements of the new_Constr_mtx has been already added
    % or cause instability (added to Unstbl_list) or that 
    % a solution has not been found
    temp_Constr_mtx=[];
    for idx1=1:size(new_Constr_mtx,1)
        in_constr_list=false;
        if not(isempty(Constr_mtx))
            if  (sum(double(new_Constr_mtx(idx1,1)==Constr_mtx(:,1) & new_Constr_mtx(idx1,2)==Constr_mtx(:,2))))
                in_constr_list=true;
            end
        end
        in_unstbl_list=false;
        if not(isempty(Unstbl_list))
            if  (sum(double(new_Constr_mtx(idx1,1)==Unstbl_list(:,1) & new_Constr_mtx(idx1,2)==Unstbl_list(:,2))))
                in_unstbl_list=true;
            end
        end
        if not(in_constr_list) && not(in_unstbl_list)
            temp_Constr_mtx_old=temp_Constr_mtx;
            temp_Constr_mtx=[temp_Constr_mtx_old;  new_Constr_mtx(idx1,1) new_Constr_mtx(idx1,2) ];
            clear temp_Constr_mtx_old
        end 
    end
    
    
    if not(isempty(temp_Constr_mtx))
        new_temp_Constr_mtx=[Constr_mtx;temp_Constr_mtx];

        % identification by using the new list of
        % coefficient constraints
        [ID_LS ID_CTLS] = ls_ctls_zero_st(Meas_mtx,B_vct,sign_constr_list,new_temp_Constr_mtx);
        unstbl_mtx=ID_CTLS.unstb;
        norm_est_err_dy=ID_CTLS.err_dy;
        
        if (unstbl_mtx||logical(norm_est_err_dy>est_err_cell{ident_num}+est_err_cell{ident_num}*th_er))
            % determine the elements that trigger the instability of the identified
            % system or that a solution has not been found
            Unstbl_list_old=Unstbl_list;
            Unstbl_list=[Unstbl_list_old; temp_Constr_mtx];
            clear Unstbl_list_old
            unstbl_link_added=0; % flag, true if for at least one edge of the 
                                 % Unstbl_list a solution has been found
            
            % define a novel constr list (add an edge at a time)
            % and then check if a solution has been found
            for idx=1:size(Unstbl_list,1)
                
                [ID_LS_1 ID_CTLS_1] =ls_ctls_zero_st(Meas_mtx,B_vct,sign_constr_list,[Constr_mtx;Unstbl_list(idx,:)]);
                norm_est_err_dy_1=ID_CTLS_1.err_dy;
                unstbl_mtx_1=ID_CTLS_1.unstb;

                if (not(unstbl_mtx_1)&&(norm_est_err_dy_1<=est_err_cell{ident_num}+est_err_cell{ident_num}*th_er))
                    % for the added edge a solution of the problem has been
                    % found
                    unstbl_link_added=1;
                    
                    % Add it (the edge) in the matrix of the added edges
                    % (update Constr_mtx)
                    Constr_mtx_old=Constr_mtx;
                    Constr_mtx=[Constr_mtx_old;Unstbl_list(idx,:)];
                    clear Constr_mtx_old
                end
            end
            if unstbl_link_added
                % update the Unstable list, for at least one edge of the 
                % Unstbl_list a solution has been found
                Unstbl_list=[];
                
                % identification by using the edges that a 
                % solution has been found
                [ID_LS ID_CTLS] = ls_ctls_zero_st(Meas_mtx,B_vct,sign_constr_list,Constr_mtx);
                
                
                ident_num=1+ident_num;% increase the number of the identifications
                % store the identified variables
                A_id_cell{ident_num}=ID_CTLS.A;
                U_id_cell{ident_num}=ID_CTLS.B;
                est_err_cell{ident_num}=ID_CTLS.err_dy;
                
                % recompute the preferential attachment function
                pref_attach=zeros(1,n_nodes);
                for idx2=1:n_nodes
                    pref_attach(idx2)=(nnz(ID_CTLS.A(:,idx2)))/...
                        (nnz(ID_CTLS.A));
                end
                % recompute the normalized weighted A matrix
                Normal_Weight_A_id=zeros(n_nodes,n_nodes);
                for idx1=1:n_nodes
                    norm_prob=pref_attach*Normal_A_id(idx1,:)';
                    for idx2=1:n_nodes
                        Normal_Weight_A_id(idx1,idx2)=Normal_A_id(idx1,idx2)*pref_attach(idx2)/norm_prob;
                    end
                end
                % Don't take account the diagonal coefficient and the prior constraints to
                % generate the edges list to be added
                for idx2=1:n_nodes
                    Normal_Weight_A_id(idx2,idx2)=0;
                end
                if not(isempty(sign_constr_list))
                    for idx1=1:size(sign_constr_list,1)
                        Normal_Weight_A_id(sign_constr_list(idx1,1),sign_constr_list(idx1,2))=0;
                    end
                end

                % recompute the edge ranking matrix (Constr_list)
                [idx_row idx_col val_coef]=find(Normal_Weight_A_id);
                [val_coef_sort idx_sort]=sort(val_coef,'descend');
                Constr_list=[idx_row(idx_sort) idx_col(idx_sort) val_coef_sort];             
            end
        else
            % the solution has been found, then redefine the Unstb_list
            Unstbl_list=[];
            ident_num=1+ident_num;% increase the number of the identifications
            
            % store the identified variables
            A_id_cell{ident_num}=ID_CTLS.A;
            U_id_cell{ident_num}=ID_CTLS.B;
            est_err_cell{ident_num}=ID_CTLS.err_dy;
            
            % update the list of added edges 
            Constr_mtx=new_temp_Constr_mtx;
            
            % recompute the preferential attachment function
            pref_attach=zeros(1,n_nodes);
            for idx2=1:n_nodes
                pref_attach(idx2)=(nnz(ID_CTLS.A(:,idx2)))/...
                    (nnz(ID_CTLS.A));
            end
            % recompute the normalized weighted A matrix
            Normal_Weight_A_id=zeros(n_nodes,n_nodes);
            for idx1=1:n_nodes
                norm_prob=pref_attach*Normal_A_id(idx1,:)';
                for idx2=1:n_nodes
                    Normal_Weight_A_id(idx1,idx2)=Normal_A_id(idx1,idx2)*pref_attach(idx2)/norm_prob;
                end
            end
            % Don't take account the diagonal coefficient and the prior constraints to
            % generate the edges list to be add added
            for idx2=1:n_nodes
                Normal_Weight_A_id(idx2,idx2)=0;
            end
            if not(isempty(sign_constr_list))
                for idx1=1:size(sign_constr_list,1)
                    Normal_Weight_A_id(sign_constr_list(idx1,1),sign_constr_list(idx1,2))=0;
                end
            end

            % recompute the edge ranking matrix (Constr_list)
            [idx_row idx_col val_coef]=find(Normal_Weight_A_id);
            [val_coef_sort idx_sort]=sort(val_coef,'descend');
            Constr_list=[idx_row(idx_sort) idx_col(idx_sort) val_coef_sort];
        end
        clear temp_Constr_mtx new_Constr_mtx new_temp_Constr_mtx
    end
    iter_num=iter_num+1
end

if ident_num==1
    
    A_id_cell{2}=A_id_temp;
end

